@@ -1,11 +1,11 @@
 """Top-level package for PyCMTensor."""
 
 __author__ = """Melvin Wong"""
-__version__ = "1.2.0"
+__version__ = "1.2.1"
 
 import numpy as np
 
-from .config import Config
+from .config import Config, about
 from .logger import *
 
 config = Config()
